<?php
#
#    Copyright (C) 2016 Dang Duong
#
#    This file is part of Caro Web.
#
#    Caro Web is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Caro Web is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with Caro Web.  If not, see <http://www.gnu.org/licenses/>.
#
defined('BASEPATH') OR exit('No direct script access allowed');

class Game extends CI_Controller {

        public function __construct()
        {
                parent::__construct();
                $this->load->helper('security');
                $this->load->model('Game_model');
                $this->load->model('Player_model');
        }

        public function index(){
                echo json_encode(array(
                        'status' => EXIT_SUCCESS,
                        'data' => $this->Game_model->getGamesList()
                ), JSON_HEX_TAG | JSON_HEX_APOS | JSON_HEX_QUOT | JSON_HEX_AMP | JSON_FORCE_OBJECT);
        }

        public function create_game()
        {
                $playerId = $this->session->userdata('player_id');
                if ($playerId > 0){
                        $gameId = $this->Game_model->createGame($playerId);
                        if ($gameId > 0)
                                echo json_encode(array(
                                        'status' => EXIT_SUCCESS,
                                        'data' => array(
                                                'gameId' => $gameId
                                        )
                                ), JSON_HEX_TAG | JSON_HEX_APOS | JSON_HEX_QUOT | JSON_HEX_AMP | JSON_FORCE_OBJECT);
                        else
                                echo json_encode(array(
                                        'status' => EXIT_ERROR,
                                        'data' => array(
                                                'errorMessage' => "Error while creating game!"
                                        )
                                ), JSON_HEX_TAG | JSON_HEX_APOS | JSON_HEX_QUOT | JSON_HEX_AMP | JSON_FORCE_OBJECT);
                }
                else
                        echo json_encode(array(
                                'status' => EXIT_ERROR,
                                'data' => array(
                                        'errorMessage' => 'You are not logged in!'
                                )
                        ), JSON_HEX_TAG | JSON_HEX_APOS | JSON_HEX_QUOT | JSON_HEX_AMP | JSON_FORCE_OBJECT);
        }

        public function join_game()
        {
                $playerId = $this->session->userdata('player_id');
                $gameId = $this->input->post('gameId');
                if ($playerId > 0 && $gameId > 0){

                        if ($this->Game_model->joinGame($gameId, $playerId) > 0){
                                $this->db->select('name');
                                $query = $this->db->get_where(CARO_DB_PREFIX.'players', array('id' => $playerId));
                                if ($query->num_rows() > 0){
                                        $player = $query->row_array();
                                        echo json_encode(array(
                                                'status' => EXIT_SUCCESS,
                                                'data' => array()
                                        ), JSON_HEX_TAG | JSON_HEX_APOS | JSON_HEX_QUOT | JSON_HEX_AMP | JSON_FORCE_OBJECT);
                                }
                                else 
                                        echo json_encode(array(
                                                'status' => EXIT_ERROR,
                                                'data' => array(
                                                        'errorMessage' => "Error while getting player!"
                                                )
                                        ), JSON_HEX_TAG | JSON_HEX_APOS | JSON_HEX_QUOT | JSON_HEX_AMP | JSON_FORCE_OBJECT);
                        }
                }
                else
                        echo json_encode(array(
                                'status' => EXIT_ERROR,
                                'data' => array(
                                        'errorMessage' => "Game or player is not specified!"
                                ) 
                        ), JSON_HEX_TAG | JSON_HEX_APOS | JSON_HEX_QUOT | JSON_HEX_AMP | JSON_FORCE_OBJECT);
        }

        public function ready_game()
        {
                $playerId = $this->session->userdata('player_id');
                $gameId = $this->input->post('gameId');
                if ($playerId > 0 && $gameId > 0){
                        if ($this->Game_model->readyGame($gameId, $playerId) > 0){
                                $this->Game_model->startGame($gameId);
                                while (1){
                                        $countdownResult = $this->countdown();
                                        if ($countdownResult > 0)
                                                $this->Game_model->updateGameData(array('turn' => $countdownResult, 'countdown' => 20));
                                        else 
                                                break;
                                }
                                echo json_encode(array(
                                        'status' => EXIT_ERROR,
                                        'data' => array(
                                                'errorMessage' => "Game is closed!"
                                        )
                                ), JSON_HEX_TAG | JSON_HEX_APOS | JSON_HEX_QUOT | JSON_HEX_AMP | JSON_FORCE_OBJECT);
                        }
                        else
                                echo json_encode(array(
                                        'status' => EXIT_ERROR,
                                        'data' => array(
                                                'errorMessage' => "Error while getting ready!"
                                        )
                                ), JSON_HEX_TAG | JSON_HEX_APOS | JSON_HEX_QUOT | JSON_HEX_AMP | JSON_FORCE_OBJECT);

                }
                else
                        echo json_encode(array(
                                'status' => EXIT_ERROR,
                                'data' => array(
                                        'errorMessage' => "Game or Player not specified!"
                                )
                        ), JSON_HEX_TAG | JSON_HEX_APOS | JSON_HEX_QUOT | JSON_HEX_AMP | JSON_FORCE_OBJECT);

        }

        public function quit_game()
        {
                $playerId = $this->session->userdata('player_id');
                $gameId = $this->input->post('gameId');
                if ($playerId > 0 && $gameId > 0){
                        $this->Game_model->quitGame($gameId, $playerId);
                        echo json_encode(array(
                                'status' => EXIT_SUCCESS,
                                'data' => array()
                        ), JSON_HEX_TAG | JSON_HEX_APOS | JSON_HEX_QUOT | JSON_HEX_AMP | JSON_FORCE_OBJECT);
                }
                else
                        echo json_encode(array(
                                'status' => EXIT_ERROR,
                                'data' => array(
                                        'errorMessage' => "Game or Player not specified!"
                                )
                        ), JSON_HEX_TAG | JSON_HEX_APOS | JSON_HEX_QUOT | JSON_HEX_AMP | JSON_FORCE_OBJECT);
        }

        public function display_game($gameId){
                $gameData = $this->Game_model->getGameData($gameId, "player1Id, player2Id, moves, turn");

                if ($gameData){
                        $data['gameId'] = $gameId;
                        $data['player1'] = $this->Player_model->getPlayerById($gameData['player1Id'], "name");
                        $data['player2'] = $this->Player_model->getPlayerById($gameData['player2Id'], "name");
                        $data['moves'] = json_decode($gameData['moves'], true);
                        $data['turn'] = $gameData['turn'];
                        echo json_encode(array(
                                'status' => EXIT_SUCCESS,
                                'data' => $data
                        ), JSON_HEX_TAG | JSON_HEX_APOS | JSON_HEX_QUOT | JSON_HEX_AMP | JSON_FORCE_OBJECT);
                }
                else
                        echo json_encode(array(
                                'status' => EXIT_ERROR,
                                'data' => array(
                                        'errorMessage' => "Game not found!"
                                )
                        ), JSON_HEX_TAG | JSON_HEX_APOS | JSON_HEX_QUOT | JSON_HEX_AMP | JSON_FORCE_OBJECT);
        }

        public function refresh_game(){

                $gameId = $this->input->post('gameId');
                $player1Id = $this->input->post('player1Id');
                $player2Id = $this->input->post('player2Id');

                $moves = json_encode($this->input->post('moves'));
                $turn = intval($this->input->post('turn'));
                $countdown = intval($this->input->post('countdown'));

                while (1){

                        $gameData = $this->Game_model->getGameData($gameId, "player1Id, player2Id, moves, turn, countdown");

                        if ($gameData){

                                $changeData = array();                               
                                $currentTime = date("Y-m-d H:i:s");

                                $player1Id = $gameData['player1Id'];
                                $player1LastActivity = $this->Player_model->getPlayerById($player1Id, "lastActivity");

                                if (!$player1LastActivity || $currentTime - date("Y-m-d H:i:s", strtotime($player1LastActivity['lastActivity']) ) > CARO_TIMEOUT){
                                        $gameData['player1Id'] = 0;
                                }
                                else if ($this->session->userdata('player_id') == $player1Id){
                                        if ($this->Player_model->updatePlayer(array('lastActivity' => date("Y-m-d H:i:s")), $player1Id) <= 0)
                                                $gameData['player1Id'] = 0;
                                }

                                $player2Id = $gameData['player2Id'];
                                if ($player2Id > 0){

                                        $player2LastActivity = $this->Player_model->getPlayerById($player2Id, "lastActivity");
                                        if (!$player2LastActivity || $currentTime - date("Y-m-d H:i:s", strtotime($player2LastActivity['lastActivity']) ) > CARO_TIMEOUT){
                                                $gameData['player2Id'] = 0;
                                        }

                                        else if ($this->session->userdata('player_id') == $player2Id){
                                                if ($this->Player_model->updatePlayer(array('lastActivity' => date("Y-m-d H:i:s")), $player2Id) <= 0)
                                                        $gameData['player2Id'] = 0;
                                        }
                                }

                                if ($player1Id != $gameData['player1Id']){
                                        if ($gameData['player1Id'] == 0){
                                                if ($gameData['player2Id'] == 0){
                                                        $this->Game_model->deleteGame($gameId);
                                                        echo json_encode(array(
                                                                'status' => EXIT_ERROR,
                                                                'data' => array(
                                                                        'errorMessage' => "Game is closed!"
                                                                )
                                                        ), JSON_HEX_TAG | JSON_HEX_APOS | JSON_HEX_QUOT | JSON_HEX_AMP | JSON_FORCE_OBJECT);
                                                        exit();
                                                }
                                                else {

                                                        $player2Name = $this->Player_model->getPlayerById($gameData['player1Id'], "name");
                                                        if ($player2Name){
                                                                $changeData['player1Id'] = $gameData['player2Id'];
                                                                $changeData['player1Name'] = $player2Name['name'];
                                                        }

                                                        $changeData['player2Id'] = 0;
                                                        $changeData['player2Name'] = "";

                                                }
                                        }
                                }

                                else if ($player2Id != $gameData['player2Id']){
                                        $changeData['player2Id'] = $gameData['player2Id'];

                                        if ($gameData['player2Id'] == 0){
                                                $changeData['player2Name'] = "";
                                        }
                                        else {
                                                $player2Name = $this->Player_model->getPlayerById($gameData['player1Id'], "name");
                                                if ($player2Name){
                                                        $changeData['player2Name'] = $player2Name['name'];
                                                }

                                                else {
                                                        $changeData['player2Id'] = 0;
                                                        $changeData['player2Name'] = "";
                                                }

                                        }
                                }

                                else {
                                        if ($gameData['moves'] != $moves)
                                                $changeData['moves'] = json_decode($gameData['moves']);
                                        if ($gameData['turn'] != $turn)
                                                $changeData['turn'] = $gameData['turn'];
                                        if ($gameData['countdown'] != $countdown)
                                                $changeData['countdown'] = $gameData['countdown'];
                                }

                                if (!empty($changeData)){
                                        echo json_encode(array(
                                                'status' => EXIT_SUCCESS,
                                                'data' => $changeData
                                        ), JSON_HEX_TAG | JSON_HEX_APOS | JSON_HEX_QUOT | JSON_HEX_AMP | JSON_FORCE_OBJECT);
                                        exit();
                                }

                        }

                        else{
                                echo json_encode(array(
                                        'status' => EXIT_ERROR,
                                        'data' => array(
                                                'errorMessage' => "Game is closed!"
                                        )
                                ), JSON_HEX_TAG | JSON_HEX_APOS | JSON_HEX_QUOT | JSON_HEX_AMP | JSON_FORCE_OBJECT);
                                exit();
                        }
                }
        }

        public function update_move(){
                $playerId = $this->session->userdata('player_id');
                if ($playerId > 0){

                        $gameData = $this->Game_model->getGameDataByPlayer($playerId, 'gameId, player1Id, player2Id, moves, turn, countdown, status');
                        if ($gameData){
                                if ($gameData['status'] == 2){
                                        if (($gameData['player1Id'] == $playerId && $gameData['turn'] == 1) || ($gameData['player2Id'] == $playerId && $gameData['turn'] == 2)){
                                                $move_x = intval($this->input->post('moveX'));
                                                $move_y = intval($this->input->post('moveY'));
                                                $moves = json_decode($gameData['moves']);
                                                if ($moves[$moveX][$moveY] == 0 && $moveX >= 0 && $moveX < 15 && $moveY >= 0 && $moveY < 15){
                                                        $moves[$moveX][$moveY] = $gameData['turn'];
                                                        $newTurn = ($gameData['turn'] == 1) ? 1 : 2;
                                                        if ($this->Game_model->updateMove($gameId, array('moves' => json_encode($moves), 'turn' => $newTurn)))
                                                                echo json_encode(array(
                                                                        'status' => EXIT_SUCCESS,
                                                                        'data' => array(
                                                                                'new_move' => ($newTurn == 2) ? "X" : "O"
                                                                        )
                                                                ), JSON_HEX_TAG | JSON_HEX_APOS | JSON_HEX_QUOT | JSON_HEX_AMP | JSON_FORCE_OBJECT);

                                                        else
                                                                echo json_encode(array(
                                                                        'status' => EXIT_ERROR,
                                                                        'data' => array(
                                                                                'errorMessage' => "Error while making a move!"
                                                                        )
                                                                ), JSON_HEX_TAG | JSON_HEX_APOS | JSON_HEX_QUOT | JSON_HEX_AMP | JSON_FORCE_OBJECT);

                                                }

                                                else
                                                        echo json_encode(array(
                                                                'status' => EXIT_ERROR,
                                                                'data' => array(
                                                                        'errorMessage' => "Illegal move!"
                                                                ) 
                                                        ), JSON_HEX_TAG | JSON_HEX_APOS | JSON_HEX_QUOT | JSON_HEX_AMP | JSON_FORCE_OBJECT);
                                        }

                                        else
                                                echo json_encode(array(
                                                        'status' => EXIT_ERROR,
                                                        'data' => array(
                                                                'errorMessage' => "Illegal move!"
                                                        )
                                                ), JSON_HEX_TAG | JSON_HEX_APOS | JSON_HEX_QUOT | JSON_HEX_AMP | JSON_FORCE_OBJECT);

                                }

                                else
                                        echo json_encode(array(
                                                'status' => EXIT_ERROR,
                                                'data' => array(
                                                        'errorMessage' => "Game is not running!"
                                                )
                                        ), JSON_HEX_TAG | JSON_HEX_APOS | JSON_HEX_QUOT | JSON_HEX_AMP | JSON_FORCE_OBJECT);

                        }
                        else                                
                                echo json_encode(array(
                                        'status' => EXIT_ERROR,
                                        'data' => array(
                                                'errorMessage' => "Game is closed!"
                                        )
                                ), JSON_HEX_TAG | JSON_HEX_APOS | JSON_HEX_QUOT | JSON_HEX_AMP | JSON_FORCE_OBJECT);

                }
                else
                        echo json_encode(array(
                                'status' => EXIT_ERROR,
                                'data' => array(
                                        'errorMessage' => "You are not logged in!"
                                )
                        ), JSON_HEX_TAG | JSON_HEX_APOS | JSON_HEX_QUOT | JSON_HEX_AMP | JSON_FORCE_OBJECT);

        }

        private function countdown($gameId){
                $currentTime = date("Y-m-d H:i:s");
                while (1){
                        if (date("Y-m-d H:i:s") - $currentTime >= 1){
                                $currentTime = date("Y-m-d H:i:s");
                                $gameData = $this->Game_model->getGameData($gameId, 'turn, countdown');
                                if ($gameData){
                                        if ($gameData['countdown'] <= 0){
                                                return ($gameData['turn'] == 1) ? 2 : 1;
                                        }
                                        else if ($this->Game_model->updateGameData(array('countdown' => $gameData['countdown'] - 1), $gameId) <= 0){
                                                return 0;
                                        }
                                }
                                else {
                                        return 0;
                                }
                        }
                }
        }

}
